class _EventTarget:
    '''https://developer.mozilla.org/en-US/docs/Web/API/EventTarget'''
    NotImplemented


class _Node(_EventTarget):
    '''https://developer.mozilla.org/en-US/docs/Web/API/Node'''
    NotImplemented


class _Element(_Node):
    '''ref of https://developer.mozilla.org/en-US/docs/Web/API/Element'''
    NotImplemented